package info.hostip;

import java.sql.SQLException;

import info.hostip.dao.core.Citybycountry;
import info.hostip.dao.core.Countries;
import info.hostip.dao.core.Ip4;
import info.hostip.dao.mgr.CitybycountryManager;
import info.hostip.dao.mgr.CountriesManager;
import info.hostip.dao.mgr.Ip4Manager;

public class IPLocator {
	
	public static final int MASK_FIRST_OCTET = 0x8;
	public static final int MASK_SECOND_OCTET = 0x4;
	public static final int MASK_THIRD_OCTET = 0x2;
	public static final int MASK_FOURTH_OCTET = 0x1;
	
	private String city;
	private String country;
	private float longitude;
	private float latitude;

	
	private IPLocator() {} 
	
	private static long ip2long(String ipAddress) {
		long f1, f2, f3, f4;
		String tokens[] = ipAddress.split("\\.");
		if (tokens.length != 4) return -1;
		try {
			f1 = Long.parseLong(tokens[0]) << 24;
			f2 = Long.parseLong(tokens[1]) << 16;
			f3 = Long.parseLong(tokens[2]) << 8;
			f4 = Long.parseLong(tokens[3]);
			return f1+f2+f3+f4;
		} catch (Exception e) {
			return -1;
		}
	}
		
	private static String replace(String ipAddress, int pos, String value) {
		StringBuffer sbuf = new StringBuffer();
		String tokens[] = ipAddress.split("\\.");
		if (tokens.length != 4) return null;
		try {
			for(int i =0; i < tokens.length; i++) {
				pos &= (0x0f >> i);
				int v = pos >> (3-i);
				boolean t = v == 1;
				sbuf.append(t?value:tokens[i]).append((i+1)<tokens.length? ".":"");
			}
		} catch (Exception e) {
			return null;
		}
		return sbuf.toString();
	}

	
	private String long2ip(long n) {
		return ((n >> 24)&0xff)+"."+((n >> 16)&0xff)+ "."+((n >> 8)&0xff)+"."+(n&0xff);
	}
	
	public static IPLocator[] locate(String ip) throws SQLException {
		long from = ip2long(replace(ip,MASK_FOURTH_OCTET,"0"));
		long to = ip2long(replace(ip,MASK_FOURTH_OCTET,"255"));
		
		Ip4Manager ip4Mgr = new Ip4Manager();
		Ip4[] ips = ip4Mgr.loadByWhere("where ip between "+from+" AND "+to);

		IPLocator[] ipg = new IPLocator[ips.length];
		CountriesManager cmgr = new CountriesManager();
		CitybycountryManager ccmgr = new CitybycountryManager();
		for(int i = 0; i < ips.length; i++) {
			
			Countries country = cmgr.loadByKey(ips[i].getCountry());
			Citybycountry city = ccmgr.loadByKey(ips[i].getCity());
			
			ipg[i] = new IPLocator();
			if(city!=null) {
				ipg[i].setCity(city.getName());
				ipg[i].setLatitude((float)city.getLat());
				ipg[i].setLongitude((float)city.getLng());
			}
			if(country!=null) {
				ipg[i].setCountry(country.getName());
			}
		}
		
		return ipg;
		
	}
	
	
	/**
	 * For unit testing purposes only
	 * 
	 * @param args
	 */
	public static void main(String args[]) {
		try {
			
			IPLocator[] ipg = IPLocator.locate("12.215.42.19");
			for(int i = 0; i < ipg.length; i++) {
				System.out.println("City: "+ipg[i].getCity() +"\nCountry: " + ipg[i].getCountry() + "\nLatitude: " + ipg[i].getLatitude()+ "\nLongitude: " + ipg[i].getLongitude());
			}
		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * @return the city
	 */
	public String getCity() {
		return city;
	}

	/**
	 * @param city the city to set
	 */
	public void setCity(String city) {
		this.city = city;
	}

	/**
	 * @return the country
	 */
	public String getCountry() {
		return country;
	}

	/**
	 * @param country the country to set
	 */
	public void setCountry(String country) {
		this.country = country;
	}

	/**
	 * @return the longitude
	 */
	public float getLongitude() {
		return longitude;
	}

	/**
	 * @param longitude the longitude to set
	 */
	public void setLongitude(float longitude) {
		this.longitude = longitude;
	}

	/**
	 * @return the latitude
	 */
	public float getLatitude() {
		return latitude;
	}

	/**
	 * @param latitude the latitude to set
	 */
	public void setLatitude(float latitude) {
		this.latitude = latitude;
	}
}
